@@ -345,10 +345,10 @@ def validate_signature_fields(sig, mandatory_fields=[b'v', b'a', b'b', b'bh', b'
         if x_sign < now - slop:
             raise ValidationError(
                 "x= value is past (%s)" % sig[b'x'])
-            if x_sign < t_sign:
-                raise ValidationError(
-                    "x= value is less than t= value (x=%s t=%s)" %
-                    (sig[b'x'], sig[b't']))
+        if x_sign < t_sign:
+            raise ValidationError(
+                "x= value is less than t= value (x=%s t=%s)" %
+                (sig[b'x'], sig[b't']))
 
 
 def rfc822_parse(message):
@@ -1052,28 +1052,26 @@ class ARC(DomainSigner):
     # extract, parse, filter & group AR headers
     ar_headers = [res.strip() for [ar, res] in self.headers if ar == b'Authentication-Results']
 
-    grouped_headers = []
+    parsed_ar_headers = []
     for res in ar_headers:
         try: # see LP: #1884044
-            grouped_headers.append((res, authres.AuthenticationResultsHeader.parse('Authentication-Results: ' + res.decode('utf-8'))))
+            # Note: parsing headers currently strips embedded comments
+            parsed_ar_headers.append(authres.AuthenticationResultsHeader.parse('Authentication-Results: ' + res.decode('utf-8')))
         except authres.core.SyntaxError:
             # Skip over invalid AR header fields
             pass
-    auth_headers = [res for res in grouped_headers if res[1].authserv_id == srv_id.decode('utf-8')]
+    auth_headers = [header for header in parsed_ar_headers if header.authserv_id == srv_id.decode('utf-8')]
 
     if len(auth_headers) == 0:
       self.logger.debug("no AR headers found, chain terminated")
       return []
 
     # consolidate headers
-    results_lists = [raw.replace(srv_id + b';', b'').strip() for (raw, parsed) in auth_headers]
-    results_lists = [tags.split(b';') for tags in results_lists]
-    results = [tag.strip() for sublist in results_lists for tag in sublist]
-    auth_results = srv_id + b'; ' + (b';' + self.linesep + b'  ').join(results)
+    results = [res for header in auth_headers for res in header.results]
+    auth_results = srv_id + b''.join(b';' + self.linesep + b' ' + str(res).encode('utf-8') for res in results)
 
     # extract cv
-    parsed_auth_results = authres.AuthenticationResultsHeader.parse('Authentication-Results: ' + auth_results.decode('utf-8'))
-    arc_results = [res for res in parsed_auth_results.results if res.method == 'arc']
+    arc_results = [res for res in results if res.method == 'arc']
     if len(arc_results) == 0:
       chain_validation_status = CV_None
     elif len(arc_results) != 1:
@@ -1120,16 +1118,13 @@ class ARC(DomainSigner):
       arc_headers = []
 
     # Compute ARC-Authentication-Results
-    aar_value = ("i=%d; " % instance).encode('utf-8') + auth_results
-    if aar_value[-1] != b'\n': aar_value += b'\r\n'
-
+    aar_value = ("i=%d; " % instance).encode('utf-8') + auth_results.rstrip() + self.linesep
+    canon_policy = CanonicalizationPolicy.from_c_value(b'relaxed/relaxed')
     new_arc_set.append(b"ARC-Authentication-Results: " + aar_value)
-    self.headers.insert(0, (b"arc-authentication-results", aar_value))
     arc_headers.insert(0, (b"ARC-Authentication-Results", aar_value))
+    self.headers = canon_policy.canonicalize_headers(arc_headers[:1]) + self.headers
 
     # Compute bh=
-    canon_policy = CanonicalizationPolicy.from_c_value(b'relaxed/relaxed')
-
     self.hasher = HASH_ALGORITHMS[self.signature_algorithm]
     h = HashThrough(self.hasher(), self.debug_content)
     h.update(canon_policy.canonicalize_body(self.body))
@@ -1157,8 +1152,8 @@ class ARC(DomainSigner):
                           b"ARC-Message-Signature", pk, standardize)
 
     new_arc_set.append(b"ARC-Message-Signature: " + res)
-    self.headers.insert(0, (b"ARC-Message-Signature", res))
     arc_headers.insert(0, (b"ARC-Message-Signature", res))
+    self.headers = canon_policy.canonicalize_headers(arc_headers[:1]) + self.headers
 
     # Compute ARC-Seal
     as_fields = [x for x in [
@@ -1186,8 +1181,8 @@ class ARC(DomainSigner):
                            b"ARC-Seal", pk, standardize)
 
     new_arc_set.append(b"ARC-Seal: " + res)
-    self.headers.insert(0, (b"ARC-Seal", res))
     arc_headers.insert(0, (b"ARC-Seal", res))
+    self.headers = canon_policy.canonicalize_headers(arc_headers[:1]) + self.headers
 
     new_arc_set.reverse()
 
